from .Potential_H99 import H99_potential

import numpy as np
def dynamics_calc_H99(xyz:np.ndarray)->np.ndarray:
    '''
    In: [N,6] np array of [x,y,z,vx,vy,vz]
    Out: [N,3] np array of [E,Lz,Lperp]
    '''
    pos, vel = xyz[:,:3], xyz[:,3:]
    K = 0.5 * np.sum(vel**2, axis=1)
    U = H99_potential(pos)
    En = U+K
    Lvec = np.cross(vel, pos)
    Lz = Lvec[:, 2]
    Lperp = np.sqrt((Lvec[:,:2]**2).sum(axis=-1))
    vec_ELzLp = np.column_stack((En,Lz,Lperp))
    return vec_ELzLp




import numpy as np

################################################################################
# ORIGINAL FILE BY JOVAN VELJANOSKI
################################################################################

G = 4.301*10**-6   # fixed GMsun [kpc km^2/s^2]

M200 = 1e12
rs = 21.5
Rvir = 258

disc_a = 6.5
disc_b = 0.26
Mdisc = 9.3*10**+10

bulge_c = 0.7
Mbulge = 3.0*10**+10

def H99_potential(pos):
    #pos galactic position, kpc, Nx3
    phi_halo  = potential_halo(pos) #NFW
    phi_disc  = potential_disc(pos) #Nig?
    phi_bulge = potential_bulge(pos) #standard
    phi_total = phi_disc + phi_bulge + phi_halo
    return phi_total
################################################################################

def potential_halo(pos):
    c = Rvir/rs
    phi_0 = G*M200/Rvir / (np.log(1+c)-c/(1+c))*c
    r = np.linalg.norm(pos,axis=1)
    phi_halo = - phi_0 * rs/r * np.log(1.0 + r/rs)
    return phi_halo



def potential_disc(pos):
    '''
    Calculates the potential due to the disc
    '''
    # parameters
    GMd = G * Mdisc

    x, y, z = pos[:,0], pos[:,1], pos[:,2]

    sqd = np.sqrt(z**2.0 + disc_b**2.0)
    # square root of the density, probably
    sqden1 = np.sqrt(x**2. + y**2.0 + (disc_a+sqd)**2.0)
    # the potential of the disc
    phi_d = -GMd/sqden1

    return phi_d

################################################################################


def potential_bulge(pos):
    '''
    Calculates the potential contribution due to the bulge
    '''
    # parameters
    GMb = G * Mbulge

    # radial distance
    r = np.linalg.norm(pos,axis=1)

    # the potential due to the bulge
    phi_b = -GMb/(r+bulge_c)

    return phi_b

################################################################################






from scipy.linalg import eigh
import numpy as np

def find_psd_U_std(covar, N_std=1) -> np.ndarray:
    """
    for scipy.stats._mutlivariate, simplified
    """
    s, u = eigh(covar, lower=True)
    s_pinv = 1 / s
    # U = np.multiply(u, np.sqrt(s_pinv))
    U = np.multiply(u, np.sqrt(np.abs(s_pinv)))
    if N_std!=1:
        U = U / N_std
    return U


def find_mahalanobis(covar, X) -> np.ndarray:
    # https://github.com/scipy/scipy/blob/v1.8.0/scipy/stats/_multivariate.py
    # Use Modified Scipy to find Mahalanobis distance Fast
    psd_U = find_psd_U_std(covar, N_std=1)
    maha = np.sqrt(np.sum(np.square(np.dot(X, psd_U)), axis=-1))
    return maha

def maha_dis_to_groups(X, group_mean, group_covar) -> np.ndarray:
    n_points = np.shape(X)[0]
    n_groups = group_mean.shape[0]
    d = np.zeros((n_points, n_groups))
    for i in range(n_groups):
        mean, covar = group_mean[i,:], group_covar[i,:]
        d[:, i] = find_mahalanobis(covar, X - mean[None, :])
    return d


def add_maha_members_to_groups(X, group_mean,
                                 group_covar, max_dis=2.13) -> np.ndarray:
    '''
    Returns group labels of the stars.
    Stars below the specified distance cut are labled.
    Stars above are given the fluff label -1
    '''
    dis = maha_dis_to_groups(X,  group_mean, group_covar)
    labels = np.argmin(dis, axis=1)
    N= dis.shape[0]
    closest_dis = dis[np.arange(N), labels]
    labels[closest_dis>max_dis] = -1
    return labels



import numpy as np
from pkg_resources import resource_filename
# data_folder =  resource_filename(__name__,"../Data/")
# data_folder =  resource_filename(__name__,"../Data/")
data_folder =  resource_filename(__name__,"/data/")
test_data_file = data_folder + "test_data.csv"
# Load Fits and Groups
named_Groups = np.load(data_folder+"original_Dodd23_Groups.npy")
group_mean = np.load(data_folder+"original_Dodd23_Group_Mean_ELzLp.npy")
group_covar = np.load(data_folder+"original_Dodd23_Group_Covar_ELzLp.npy")



from astropy.coordinates import SkyCoord
import astropy.units as u
from astropy.coordinates import Galactocentric
from astropy.table import Table, QTable
import numpy as np


def add_units(data:dict|Table|QTable):
    data["ra"]=data["ra"] if isinstance(data["ra"],u.Quantity) else data["ra"]*u.degree
    data["dec"]=data["dec"] if isinstance(data["dec"],u.Quantity) else data["dec"]*u.degree
    data["distance"]=data["distance"] if isinstance(data["distance"],u.Quantity) else data["distance"]*u.kpc
    data["pmra"] = data["pmra"] if isinstance(data["pmra"] ,u.Quantity) else data["pmra"]*(u.mas/u.yr)
    data["pmdec"] = data["pmdec"] if isinstance(data["pmdec"] ,u.Quantity) else data["pmdec"]*(u.mas/u.yr)
    data["radial_velocity"] = data["radial_velocity"] if isinstance(data["radial_velocity"] ,u.Quantity) else data["radial_velocity"]*(u.km/u.s)
    return data

def coord_transform_icrs_Galacto(data:dict)->np.ndarray:
    data = add_units(data)
    v_sun = [11.1, 232.8 +12.24, 7.25] * (u.km / u.s)  # [vx, vy, vz]
    gc_frame = Galactocentric(
        galcen_distance=8.20 * u.kpc, galcen_v_sun=v_sun, z_sun=0 * u.pc
    )

    coords=SkyCoord(ra=data["ra"], dec=data["dec"], pm_ra_cosdec=data["pmra"],
                         pm_dec=data["pmdec"],
                         radial_velocity=data["radial_velocity"], distance=data["distance"],
                         frame='icrs').transform_to(gc_frame)
    n_points = len(data["distance"])
    xyz = np.empty((n_points,6))
    xyz[:,:3] = np.asarray(coords.cartesian.xyz.to(u.kpc)).T
    xyz[:,3:] = np.asarray(coords.velocity.d_xyz.to(u.km/u.s)).T
    return xyz
